@@ -16,6 +16,7 @@
 """The cgsnapshots api."""
 
 from oslo_log import log as logging
+import six
 import webob
 from webob import exc
 
@@ -23,63 +24,13 @@ from cinder.api import common
 from cinder.api import extensions
 from cinder.api.openstack import wsgi
 from cinder.api.views import cgsnapshots as cgsnapshot_views
-from cinder.api import xmlutil
 from cinder import consistencygroup as consistencygroupAPI
 from cinder import exception
 from cinder.i18n import _, _LI
-from cinder import utils
 
 LOG = logging.getLogger(__name__)
 
 
-def make_cgsnapshot(elem):
-    elem.set('id')
-    elem.set('consistencygroup_id')
-    elem.set('status')
-    elem.set('created_at')
-    elem.set('name')
-    elem.set('description')
-
-
-class CgsnapshotTemplate(xmlutil.TemplateBuilder):
-    def construct(self):
-        root = xmlutil.TemplateElement('cgsnapshot', selector='cgsnapshot')
-        make_cgsnapshot(root)
-        alias = Cgsnapshots.alias
-        namespace = Cgsnapshots.namespace
-        return xmlutil.MasterTemplate(root, 1, nsmap={alias: namespace})
-
-
-class CgsnapshotsTemplate(xmlutil.TemplateBuilder):
-    def construct(self):
-        root = xmlutil.TemplateElement('cgsnapshots')
-        elem = xmlutil.SubTemplateElement(root, 'cgsnapshot',
-                                          selector='cgsnapshots')
-        make_cgsnapshot(elem)
-        alias = Cgsnapshots.alias
-        namespace = Cgsnapshots.namespace
-        return xmlutil.MasterTemplate(root, 1, nsmap={alias: namespace})
-
-
-class CreateDeserializer(wsgi.MetadataXMLDeserializer):
-    def default(self, string):
-        dom = utils.safe_minidom_parse_string(string)
-        cgsnapshot = self._extract_cgsnapshot(dom)
-        return {'body': {'cgsnapshot': cgsnapshot}}
-
-    def _extract_cgsnapshot(self, node):
-        cgsnapshot = {}
-        cgsnapshot_node = self.find_first_child_named(node, 'cgsnapshot')
-
-        attributes = ['name',
-                      'description']
-
-        for attr in attributes:
-            if cgsnapshot_node.getAttribute(attr):
-                cgsnapshot[attr] = cgsnapshot_node.getAttribute(attr)
-        return cgsnapshot
-
-
 class CgsnapshotsController(wsgi.Controller):
     """The cgsnapshots API controller for the OpenStack API."""
 
@@ -89,18 +40,15 @@ class CgsnapshotsController(wsgi.Controller):
         self.cgsnapshot_api = consistencygroupAPI.API()
         super(CgsnapshotsController, self).__init__()
 
-    @wsgi.serializers(xml=CgsnapshotTemplate)
     def show(self, req, id):
         """Return data about the given cgsnapshot."""
         LOG.debug('show called for member %s', id)
         context = req.environ['cinder.context']
 
-        try:
-            cgsnapshot = self.cgsnapshot_api.get_cgsnapshot(
-                context,
-                cgsnapshot_id=id)
-        except exception.CgSnapshotNotFound as error:
-            raise exc.HTTPNotFound(explanation=error.msg)
+        # Not found exception will be handled at the wsgi level
+        cgsnapshot = self.cgsnapshot_api.get_cgsnapshot(
+            context,
+            cgsnapshot_id=id)
 
         return self._view_builder.detail(req, cgsnapshot)
 
@@ -109,30 +57,28 @@ class CgsnapshotsController(wsgi.Controller):
         LOG.debug('delete called for member %s', id)
         context = req.environ['cinder.context']
 
-        LOG.info(_LI('Delete cgsnapshot with id: %s'), id, context=context)
+        LOG.info(_LI('Delete cgsnapshot with id: %s'), id)
 
         try:
             cgsnapshot = self.cgsnapshot_api.get_cgsnapshot(
                 context,
                 cgsnapshot_id=id)
             self.cgsnapshot_api.delete_cgsnapshot(context, cgsnapshot)
-        except exception.CgSnapshotNotFound as error:
-            raise exc.HTTPNotFound(explanation=error.msg)
-        except exception.InvalidCgSnapshot:
-            msg = _("Invalid cgsnapshot")
-            raise exc.HTTPBadRequest(explanation=msg)
+        except exception.CgSnapshotNotFound:
+            # Not found exception will be handled at the wsgi level
+            raise
+        except exception.InvalidCgSnapshot as e:
+            raise exc.HTTPBadRequest(explanation=six.text_type(e))
         except Exception:
             msg = _("Failed cgsnapshot")
             raise exc.HTTPBadRequest(explanation=msg)
 
         return webob.Response(status_int=202)
 
-    @wsgi.serializers(xml=CgsnapshotsTemplate)
     def index(self, req):
         """Returns a summary list of cgsnapshots."""
         return self._get_cgsnapshots(req, is_detail=False)
 
-    @wsgi.serializers(xml=CgsnapshotsTemplate)
     def detail(self, req):
         """Returns a detailed list of cgsnapshots."""
         return self._get_cgsnapshots(req, is_detail=True)
@@ -150,8 +96,6 @@ class CgsnapshotsController(wsgi.Controller):
         return cgsnapshots
 
     @wsgi.response(202)
-    @wsgi.serializers(xml=CgsnapshotTemplate)
-    @wsgi.deserializers(xml=CreateDeserializer)
     def create(self, req, body):
         """Create a new cgsnapshot."""
         LOG.debug('Creating new cgsnapshot %s', body)
@@ -167,10 +111,8 @@ class CgsnapshotsController(wsgi.Controller):
             msg = _("'consistencygroup_id' must be specified")
             raise exc.HTTPBadRequest(explanation=msg)
 
-        try:
-            group = self.cgsnapshot_api.get(context, group_id)
-        except exception.ConsistencyGroupNotFound as error:
-            raise exc.HTTPNotFound(explanation=error.msg)
+        # Not found exception will be handled at the wsgi level
+        group = self.cgsnapshot_api.get(context, group_id)
 
         name = cgsnapshot.get('name', None)
         description = cgsnapshot.get('description', None)
@@ -182,10 +124,9 @@ class CgsnapshotsController(wsgi.Controller):
         try:
             new_cgsnapshot = self.cgsnapshot_api.create_cgsnapshot(
                 context, group, name, description)
+        # Not found exception will be handled at the wsgi level
         except exception.InvalidCgSnapshot as error:
             raise exc.HTTPBadRequest(explanation=error.msg)
-        except exception.CgSnapshotNotFound as error:
-            raise exc.HTTPNotFound(explanation=error.msg)
 
         retval = self._view_builder.summary(req, new_cgsnapshot)
 
@@ -197,7 +138,6 @@ class Cgsnapshots(extensions.ExtensionDescriptor):
 
     name = 'Cgsnapshots'
     alias = 'cgsnapshots'
-    namespace = 'http://docs.openstack.org/volume/ext/cgsnapshots/api/v1'
     updated = '2014-08-18T00:00:00+00:00'
 
     def get_resources(self):
