@@ -13,7 +13,6 @@
 #    under the License.
 
 from oslo_config import cfg
-from oslo_log import log as logging
 from oslo_utils import versionutils
 from oslo_versionedobjects import fields
 
@@ -22,30 +21,33 @@ from cinder import exception
 from cinder.i18n import _
 from cinder import objects
 from cinder.objects import base
+from cinder.objects import fields as c_fields
+
 
 CONF = cfg.CONF
-LOG = logging.getLogger(__name__)
 
 
 @base.CinderObjectRegistry.register
 class Snapshot(base.CinderPersistentObject, base.CinderObject,
                base.CinderObjectDictCompat):
     # Version 1.0: Initial version
-    VERSION = '1.0'
+    # Version 1.1: Changed 'status' field to use SnapshotStatusField
+    VERSION = '1.1'
 
     # NOTE(thangp): OPTIONAL_FIELDS are fields that would be lazy-loaded. They
     # are typically the relationship in the sqlalchemy object.
-    OPTIONAL_FIELDS = ('volume', 'metadata', 'cgsnapshot')
+    OPTIONAL_FIELDS = ('volume', 'metadata', 'cgsnapshot', 'group_snapshot')
 
     fields = {
         'id': fields.UUIDField(),
 
-        'user_id': fields.UUIDField(nullable=True),
-        'project_id': fields.UUIDField(nullable=True),
+        'user_id': fields.StringField(nullable=True),
+        'project_id': fields.StringField(nullable=True),
 
         'volume_id': fields.UUIDField(nullable=True),
         'cgsnapshot_id': fields.UUIDField(nullable=True),
-        'status': fields.StringField(nullable=True),
+        'group_snapshot_id': fields.UUIDField(nullable=True),
+        'status': c_fields.SnapshotStatusField(nullable=True),
         'progress': fields.StringField(nullable=True),
         'volume_size': fields.IntegerField(nullable=True),
 
@@ -56,16 +58,21 @@ class Snapshot(base.CinderPersistentObject, base.CinderObject,
         'volume_type_id': fields.UUIDField(nullable=True),
 
         'provider_location': fields.StringField(nullable=True),
-        'provider_id': fields.UUIDField(nullable=True),
+        'provider_id': fields.StringField(nullable=True),
         'metadata': fields.DictOfStringsField(),
         'provider_auth': fields.StringField(nullable=True),
 
         'volume': fields.ObjectField('Volume', nullable=True),
         'cgsnapshot': fields.ObjectField('CGSnapshot', nullable=True),
+        'group_snapshot': fields.ObjectField('GroupSnapshot', nullable=True),
     }
 
+    @property
+    def service_topic_queue(self):
+        return self.volume.service_topic_queue
+
     @classmethod
-    def _get_expected_attrs(cls, context):
+    def _get_expected_attrs(cls, context, *args, **kwargs):
         return 'metadata',
 
     # NOTE(thangp): obj_extra_fields is used to hold properties that are not
@@ -128,6 +135,12 @@ class Snapshot(base.CinderPersistentObject, base.CinderObject,
             cgsnapshot._from_db_object(context, cgsnapshot,
                                        db_snapshot['cgsnapshot'])
             snapshot.cgsnapshot = cgsnapshot
+        if 'group_snapshot' in expected_attrs:
+            group_snapshot = objects.GroupSnapshot(context)
+            group_snapshot._from_db_object(context, group_snapshot,
+                                           db_snapshot['group_snapshot'])
+            snapshot.group_snapshot = group_snapshot
+
         if 'metadata' in expected_attrs:
             metadata = db_snapshot.get('snapshot_metadata')
             if metadata is None:
@@ -138,7 +151,6 @@ class Snapshot(base.CinderPersistentObject, base.CinderObject,
         snapshot.obj_reset_changes()
         return snapshot
 
-    @base.remotable
     def create(self):
         if self.obj_attr_is_set('id'):
             raise exception.ObjectActionError(action='create',
@@ -151,11 +163,17 @@ class Snapshot(base.CinderPersistentObject, base.CinderObject,
         if 'cgsnapshot' in updates:
             raise exception.ObjectActionError(action='create',
                                               reason=_('cgsnapshot assigned'))
+        if 'cluster' in updates:
+            raise exception.ObjectActionError(
+                action='create', reason=_('cluster assigned'))
+        if 'group_snapshot' in updates:
+            raise exception.ObjectActionError(
+                action='create',
+                reason=_('group_snapshot assigned'))
 
         db_snapshot = db.snapshot_create(self._context, updates)
         self._from_db_object(self._context, self, db_snapshot)
 
-    @base.remotable
     def save(self):
         updates = self.cinder_obj_get_changes()
         if updates:
@@ -165,6 +183,13 @@ class Snapshot(base.CinderPersistentObject, base.CinderObject,
             if 'cgsnapshot' in updates:
                 raise exception.ObjectActionError(
                     action='save', reason=_('cgsnapshot changed'))
+            if 'group_snapshot' in updates:
+                raise exception.ObjectActionError(
+                    action='save', reason=_('group_snapshot changed'))
+
+            if 'cluster' in updates:
+                raise exception.ObjectActionError(
+                    action='save', reason=_('cluster changed'))
 
             if 'metadata' in updates:
                 # Metadata items that are not specified in the
@@ -178,9 +203,10 @@ class Snapshot(base.CinderPersistentObject, base.CinderObject,
 
         self.obj_reset_changes()
 
-    @base.remotable
     def destroy(self):
-        db.snapshot_destroy(self._context, self.id)
+        updated_values = db.snapshot_destroy(self._context, self.id)
+        self.update(updated_values)
+        self.obj_reset_changes(updated_values.keys())
 
     def obj_load_attr(self, attrname):
         if attrname not in self.OPTIONAL_FIELDS:
@@ -199,6 +225,11 @@ class Snapshot(base.CinderPersistentObject, base.CinderObject,
             self.cgsnapshot = objects.CGSnapshot.get_by_id(self._context,
                                                            self.cgsnapshot_id)
 
+        if attrname == 'group_snapshot':
+            self.group_snapshot = objects.GroupSnapshot.get_by_id(
+                self._context,
+                self.group_snapshot_id)
+
         self.obj_reset_changes(fields=[attrname])
 
     def delete_metadata_key(self, context, key):
@@ -211,7 +242,7 @@ class Snapshot(base.CinderPersistentObject, base.CinderObject,
         if not md_was_changed:
             self.obj_reset_changes(['metadata'])
 
-    @base.remotable_classmethod
+    @classmethod
     def snapshot_data_get_for_project(cls, context, project_id,
                                       volume_type_id=None):
         return db.snapshot_data_get_for_project(context, project_id,
@@ -226,7 +257,7 @@ class SnapshotList(base.ObjectListBase, base.CinderObject):
         'objects': fields.ListOfObjectsField('Snapshot'),
     }
 
-    @base.remotable_classmethod
+    @classmethod
     def get_all(cls, context, search_opts, marker=None, limit=None,
                 sort_keys=None, sort_dirs=None, offset=None):
         snapshots = db.snapshot_get_all(context, search_opts, marker, limit,
@@ -235,14 +266,14 @@ class SnapshotList(base.ObjectListBase, base.CinderObject):
         return base.obj_make_list(context, cls(context), objects.Snapshot,
                                   snapshots, expected_attrs=expected_attrs)
 
-    @base.remotable_classmethod
+    @classmethod
     def get_by_host(cls, context, host, filters=None):
         snapshots = db.snapshot_get_by_host(context, host, filters)
         expected_attrs = Snapshot._get_expected_attrs(context)
         return base.obj_make_list(context, cls(context), objects.Snapshot,
                                   snapshots, expected_attrs=expected_attrs)
 
-    @base.remotable_classmethod
+    @classmethod
     def get_all_by_project(cls, context, project_id, search_opts, marker=None,
                            limit=None, sort_keys=None, sort_dirs=None,
                            offset=None):
@@ -253,23 +284,31 @@ class SnapshotList(base.ObjectListBase, base.CinderObject):
         return base.obj_make_list(context, cls(context), objects.Snapshot,
                                   snapshots, expected_attrs=expected_attrs)
 
-    @base.remotable_classmethod
+    @classmethod
     def get_all_for_volume(cls, context, volume_id):
         snapshots = db.snapshot_get_all_for_volume(context, volume_id)
         expected_attrs = Snapshot._get_expected_attrs(context)
         return base.obj_make_list(context, cls(context), objects.Snapshot,
                                   snapshots, expected_attrs=expected_attrs)
 
-    @base.remotable_classmethod
+    @classmethod
     def get_active_by_window(cls, context, begin, end):
         snapshots = db.snapshot_get_active_by_window(context, begin, end)
         expected_attrs = Snapshot._get_expected_attrs(context)
         return base.obj_make_list(context, cls(context), objects.Snapshot,
                                   snapshots, expected_attrs=expected_attrs)
 
-    @base.remotable_classmethod
+    @classmethod
     def get_all_for_cgsnapshot(cls, context, cgsnapshot_id):
         snapshots = db.snapshot_get_all_for_cgsnapshot(context, cgsnapshot_id)
         expected_attrs = Snapshot._get_expected_attrs(context)
         return base.obj_make_list(context, cls(context), objects.Snapshot,
                                   snapshots, expected_attrs=expected_attrs)
+
+    @classmethod
+    def get_all_for_group_snapshot(cls, context, group_snapshot_id):
+        snapshots = db.snapshot_get_all_for_group_snapshot(
+            context, group_snapshot_id)
+        expected_attrs = Snapshot._get_expected_attrs(context)
+        return base.obj_make_list(context, cls(context), objects.Snapshot,
+                                  snapshots, expected_attrs=expected_attrs)
