@@ -19,18 +19,18 @@ ISCSI volume driver for IBM Storwize family and SVC storage systems.
 
 Notes:
 1. If you specify both a password and a key file, this driver will use the
-   key file only.
+key file only.
 2. When using a key file for authentication, it is up to the user or
-   system administrator to store the private key in a safe manner.
+system administrator to store the private key in a safe manner.
 3. The defaults for creating volumes are "-rsize 2% -autoexpand
-   -grainsize 256 -warning 0".  These can be changed in the configuration
-   file or by using volume types(recommended only for advanced users).
+-grainsize 256 -warning 0".  These can be changed in the configuration
+file or by using volume types(recommended only for advanced users).
 
 Limitations:
 1. The driver expects CLI output in English, error messages may be in a
-   localized format.
+localized format.
 2. Clones and creating volumes from snapshots, where the source and target
-   are of different sizes, is not supported.
+are of different sizes, is not supported.
 
 """
 
@@ -40,6 +40,7 @@ from oslo_utils import excutils
 
 from cinder import exception
 from cinder.i18n import _, _LE, _LW
+from cinder import interface
 from cinder import utils
 
 from cinder.volume.drivers.ibm.storwize_svc import (
@@ -58,35 +59,42 @@ CONF = cfg.CONF
 CONF.register_opts(storwize_svc_iscsi_opts)
 
 
+@interface.volumedriver
 class StorwizeSVCISCSIDriver(storwize_common.StorwizeSVCCommonDriver):
     """IBM Storwize V7000 and SVC iSCSI volume driver.
 
     Version history:
-    1.0 - Initial driver
-    1.1 - FC support, create_cloned_volume, volume type support,
-          get_volume_stats, minor bug fixes
-    1.2.0 - Added retype
-    1.2.1 - Code refactor, improved exception handling
-    1.2.2 - Fix bug #1274123 (races in host-related functions)
-    1.2.3 - Fix Fibre Channel connectivity: bug #1279758 (add delim to
-            lsfabric, clear unused data from connections, ensure matching
-            WWPNs by comparing lower case
-    1.2.4 - Fix bug #1278035 (async migration/retype)
-    1.2.5 - Added support for manage_existing (unmanage is inherited)
-    1.2.6 - Added QoS support in terms of I/O throttling rate
-    1.3.1 - Added support for volume replication
-    1.3.2 - Added support for consistency group
-    1.3.3 - Update driver to use ABC metaclasses
-    2.0 - Code refactor, split init file and placed shared methods for
-          FC and iSCSI within the StorwizeSVCCommonDriver class
-    2.0.1 - Added support for multiple pools with model update
-    2.1 - Added replication V2 support to the global/metro mirror
-          mode
-    2.1.1 - Update replication to version 2.1
+
+    .. code-block:: none
+
+        1.0 - Initial driver
+        1.1 - FC support, create_cloned_volume, volume type support,
+              get_volume_stats, minor bug fixes
+        1.2.0 - Added retype
+        1.2.1 - Code refactor, improved exception handling
+        1.2.2 - Fix bug #1274123 (races in host-related functions)
+        1.2.3 - Fix Fibre Channel connectivity: bug #1279758 (add delim
+                to lsfabric, clear unused data from connections, ensure
+                matching WWPNs by comparing lower case
+        1.2.4 - Fix bug #1278035 (async migration/retype)
+        1.2.5 - Added support for manage_existing (unmanage is inherited)
+        1.2.6 - Added QoS support in terms of I/O throttling rate
+        1.3.1 - Added support for volume replication
+        1.3.2 - Added support for consistency group
+        1.3.3 - Update driver to use ABC metaclasses
+        2.0 - Code refactor, split init file and placed shared methods
+              for FC and iSCSI within the StorwizeSVCCommonDriver class
+        2.0.1 - Added support for multiple pools with model update
+        2.1 - Added replication V2 support to the global/metro mirror
+              mode
+        2.1.1 - Update replication to version 2.1
     """
 
     VERSION = "2.1.1"
 
+    # ThirdPartySystems wiki page
+    CI_WIKI_NAME = "IBM_STORWIZE_CI"
+
     def __init__(self, *args, **kwargs):
         super(StorwizeSVCISCSIDriver, self).__init__(*args, **kwargs)
         self.protocol = 'iSCSI'
@@ -101,8 +109,15 @@ class StorwizeSVCISCSIDriver(storwize_common.StorwizeSVCCommonDriver):
             raise exception.InvalidConnectorException(
                 missing='initiator')
 
-    @utils.synchronized('storwize-host', external=True)
     def initialize_connection(self, volume, connector):
+        """Perform necessary work to make an iSCSI connection."""
+        @utils.synchronized('storwize-host' + self._state['system_id'] +
+                            connector['host'], external=True)
+        def _do_initialize_connection_locked():
+            return self._do_initialize_connection(volume, connector)
+        return _do_initialize_connection_locked()
+
+    def _do_initialize_connection(self, volume, connector):
         """Perform necessary work to make an iSCSI connection.
 
         To be able to create an iSCSI connection from a given host to a
@@ -111,7 +126,7 @@ class StorwizeSVCISCSIDriver(storwize_common.StorwizeSVCCommonDriver):
         2. Create new host on the storage system if it does not yet exist
         3. Map the volume to the host if it is not already done
         4. Return the connection information for relevant nodes (in the
-           proper I/O group)
+        proper I/O group)
         """
         LOG.debug('enter: initialize_connection: volume %(vol)s with connector'
                   ' %(conn)s', {'vol': volume['id'], 'conn': connector})
@@ -199,7 +214,7 @@ class StorwizeSVCISCSIDriver(storwize_common.StorwizeSVCCommonDriver):
 
         except Exception:
             with excutils.save_and_reraise_exception():
-                self.terminate_connection(volume, connector)
+                self._do_terminate_connection(volume, connector)
                 LOG.error(_LE('initialize_connection: Failed '
                               'to collect return '
                               'properties for volume %(vol)s and connector '
@@ -213,8 +228,22 @@ class StorwizeSVCISCSIDriver(storwize_common.StorwizeSVCCommonDriver):
 
         return {'driver_volume_type': 'iscsi', 'data': properties, }
 
-    @utils.synchronized('storwize-host', external=True)
     def terminate_connection(self, volume, connector, **kwargs):
+        """Cleanup after an iSCSI connection has been terminated."""
+        # If a fake connector is generated by nova when the host
+        # is down, then the connector will not have a host property,
+        # In this case construct the lock without the host property
+        # so that all the fake connectors to an SVC are serialized
+        host = connector['host'] if 'host' in connector else ""
+
+        @utils.synchronized('storwize-host' + self._state['system_id'] + host,
+                            external=True)
+        def _do_terminate_connection_locked():
+            return self._do_terminate_connection(volume, connector,
+                                                 **kwargs)
+        return _do_terminate_connection_locked()
+
+    def _do_terminate_connection(self, volume, connector, **kwargs):
         """Cleanup after an iSCSI connection has been terminated.
 
         When we clean up a terminated connection between a given connector
@@ -222,7 +251,7 @@ class StorwizeSVCISCSIDriver(storwize_common.StorwizeSVCCommonDriver):
         1. Translate the given connector to a host name
         2. Remove the volume-to-host mapping if it exists
         3. Delete the host if it has no more mappings (hosts are created
-           automatically by this driver when mappings are created)
+        automatically by this driver when mappings are created)
         """
         LOG.debug('enter: terminate_connection: volume %(vol)s with connector'
                   ' %(conn)s', {'vol': volume['id'], 'conn': connector})
