@@ -39,27 +39,29 @@ class PrepareForQuotaReservationTask(flow_utils.CinderTask):
         self.db = db
         self.driver = driver
 
-    def execute(self, context, volume_ref, manage_existing_ref):
-        volume_id = volume_ref['id']
+    def execute(self, context, volume, manage_existing_ref):
         if not self.driver.initialized:
             driver_name = self.driver.__class__.__name__
             LOG.error(_LE("Unable to manage existing volume. "
                           "Volume driver %s not initialized.") % driver_name)
-            flow_common.error_out_volume(context, self.db, volume_id,
-                                         reason=_("Volume driver %s "
-                                                  "not initialized.") %
-                                         driver_name)
+            flow_common.error_out(volume, _("Volume driver %s not "
+                                            "initialized.") % driver_name)
             raise exception.DriverNotInitialized()
 
-        size = self.driver.manage_existing_get_size(volume_ref,
+        size = self.driver.manage_existing_get_size(volume,
                                                     manage_existing_ref)
 
         return {'size': size,
-                'volume_type_id': volume_ref['volume_type_id'],
-                'volume_properties': volume_ref,
-                'volume_spec': {'status': volume_ref['status'],
-                                'volume_name': volume_ref['name'],
-                                'volume_id': volume_ref['id']}}
+                'volume_type_id': volume.volume_type_id,
+                'volume_properties': volume,
+                'volume_spec': {'status': volume.status,
+                                'volume_name': volume.name,
+                                'volume_id': volume.id}}
+
+    def revert(self, context, result, flow_failures, volume, **kwargs):
+        reason = _('Volume manage failed.')
+        flow_common.error_out(volume, reason=reason)
+        LOG.error(_LE("Volume %s: manage failed."), volume.id)
 
 
 class ManageExistingTask(flow_utils.CinderTask):
@@ -72,26 +74,27 @@ class ManageExistingTask(flow_utils.CinderTask):
         self.db = db
         self.driver = driver
 
-    def execute(self, context, volume_ref, manage_existing_ref, size):
-        model_update = self.driver.manage_existing(volume_ref,
+    def execute(self, context, volume, manage_existing_ref, size):
+        model_update = self.driver.manage_existing(volume,
                                                    manage_existing_ref)
+
         if not model_update:
             model_update = {}
         model_update.update({'size': size})
         try:
-            volume_ref.update(model_update)
-            volume_ref.save()
+            volume.update(model_update)
+            volume.save()
         except exception.CinderException:
             LOG.exception(_LE("Failed updating model of volume %(volume_id)s"
                               " with creation provided model %(model)s") %
-                          {'volume_id': volume_ref['id'],
+                          {'volume_id': volume.id,
                            'model': model_update})
             raise
 
-        return {'volume': volume_ref}
+        return {'volume': volume}
 
 
-def get_flow(context, db, driver, host, volume_id, ref):
+def get_flow(context, db, driver, host, volume, ref):
     """Constructs and returns the manager entrypoint flow."""
 
     flow_name = ACTION.replace(":", "_") + "_manager"
@@ -102,13 +105,12 @@ def get_flow(context, db, driver, host, volume_id, ref):
     # determined.
     create_what = {
         'context': context,
-        'volume_id': volume_id,
+        'volume': volume,
         'manage_existing_ref': ref,
-        'optional_args': {'is_quota_committed': False}
+        'optional_args': {'is_quota_committed': False},
     }
 
-    volume_flow.add(create_mgr.ExtractVolumeRefTask(db, host),
-                    create_mgr.NotifyVolumeActionTask(db,
+    volume_flow.add(create_mgr.NotifyVolumeActionTask(db,
                                                       "manage_existing.start"),
                     PrepareForQuotaReservationTask(db, driver),
                     create_api.QuotaReserveTask(),
