@@ -67,13 +67,16 @@ class CiscoFCZoneClientCLI(object):
         are active then it will return empty map.
 
         :returns: Map -- active zone set map in the following format
-        {
-            'zones':
-                {'openstack50060b0000c26604201900051ee8e329':
-                    ['50060b0000c26604', '201900051ee8e329']
-                },
-            'active_zone_config': 'OpenStack_Cfg'
-        }
+
+        .. code-block:: python
+
+            {
+                'zones':
+                    {'openstack50060b0000c26604201900051ee8e329':
+                        ['50060b0000c26604', '201900051ee8e329']
+                    },
+                'active_zone_config': 'OpenStack_Cfg'
+            }
         """
         zone_set = {}
         zone = {}
@@ -135,15 +138,28 @@ class CiscoFCZoneClientCLI(object):
         """Add zone configuration.
 
         This method will add the zone configuration passed by user.
-            input params:
-            zones - zone names mapped to members and VSANs.
-            zone members are colon separated but case-insensitive
+
+        :param zones: Zone names mapped to members and VSANs
+                      Zone members are colon separated but case-insensitive
+
+        .. code-block:: python
+
             {   zonename1:[zonememeber1,zonemember2,...],
                 zonename2:[zonemember1, zonemember2,...]...}
-            e.g: {'openstack50060b0000c26604201900051ee8e329':
-                    ['50:06:0b:00:00:c2:66:04', '20:19:00:05:1e:e8:e3:29']
-                }
-            activate - True/False
+
+            e.g:
+
+            {
+                'openstack50060b0000c26604201900051ee8e329':
+                  ['50:06:0b:00:00:c2:66:04', '20:19:00:05:1e:e8:e3:29']
+            }
+
+        :param activate: True will activate the zone config.
+        :param fabric_vsan:
+        :param active_zone_set: Active zone set dict retrieved from
+                                get_active_zone_set method
+        :param zone_status: Status of the zone
+        :raises: CiscoZoningCliException
         """
         LOG.debug("Add Zones - Zones passed: %s", zones)
 
